using System;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;
using Gma.System.MouseKeyHook;

/// Kenrick Mock
/// Mouse attention-highlighter
/// TO DO: Comment and re-factor this messy code
/// 
/// THIS VERSION: Ink/Stylus namespaces gone so it runs on any Windows or Vista
/// machine without a tablet or pen (or Wacom external tablet)
/// Latest version eliminates DLL and uses in-code mousehook.  Check high bit code for
/// pen/touch or mouse.  
/// Extended screen removed and scaling removed.  Uses Windows API call to get physical cursor location
/// so it works correctly on extended screens and different screen magnifications.

namespace PenAttention
{
    public partial class frmAttention : Form
    {
        private IKeyboardMouseEvents m_Events;
        public static bool pen_detected = false;    // Whether or not pen is nearby
        public enum Attention_Mode { Highlight, Pencil, Pointer, None };
        private static Attention_Mode attention_type = Attention_Mode.Highlight;
        long lMsg;
        //IntPtr myHwnd;
        PerPixelAlphaForm highlightForm = new PerPixelAlphaForm();
        private static Bitmap curBmp = null;
        private bool circleHighlight = true;    // Default is we are using the circle highlight
        private bool made_changes = false;      // Whether or not we changed a setting
        private bool ignoreClick = true;        // Weird issue x/y coordinates jump when mouse down?
        private int lastX = 0;                  // Last x/y coordinates
        private int lastY = 0;

        // Right click message
        //private static int RCLICKCODE = 32767;

        // Keyboard on/off toggle
        private static int KEYTOGGLE = 0x0312;
        private bool highlightChecked = true;      // Remember previous settings
        private bool pencilChecked = false;
        private bool pointerChecked = false;

        // Hotkey settings
        private static int CTRL_ALT_F9 = 2;		// Codes for CTRl_ALT_F9 or just CTRL_F9
        private static int CTRL_F9 = 3;
        private static int HOTKEY_OFF = 4;  	// Code for hotkey off
        private int hotkey = CTRL_ALT_F9;
        private int origHotkey;

        // Original highlight settings
        private int origRadius, origOpacity;
        private Color origColor;
        private bool origHighlightMouse;
        private bool origCircleHighlight;
        private int origHeight, origWidth;
        private bool origToggleColor;

        [DllImport("user32.dll", SetLastError = true)]
        static extern bool ChangeWindowMessageFilter(uint message, uint dwFlag);

        [DllImport("user32.dll")]
        static extern int SetWindowLong(IntPtr hWnd, int nIndex, int dwNewLong);

        [DllImport("user32.dll")]
        static extern long GetWindowLong(IntPtr hWnd, int nIndex);

        /*
        [DllImport("MouseHook.dll")]
        static extern bool setMyHook(IntPtr hWnd);

        [DllImport("MouseHook.dll")]
        static extern bool clearMyHook(IntPtr hWnd);

        [DllImport("MouseHook.dll")]
        static extern void setFlag(byte flag);
        */

        [DllImport("user32.dll", EntryPoint = "FindWindow", SetLastError = true)]
        static extern System.IntPtr FindWindowByCaption(int ZeroOnly, string lpWindowName);

        [DllImport("user32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        static extern uint RegisterWindowMessage(string lpString);

        [DllImport("user32.dll")]
        public static extern bool RegisterHotKey(IntPtr hWnd,
          int id, int fsModifiers, int vlc);

        [DllImport("user32.dll")]
        public static extern bool UnregisterHotKey(IntPtr hWnd, int id);

        // To determine screen scaling
        public enum DeviceCap
        {
            VERTRES = 10,
            DESKTOPVERTRES = 117,
            // http://pinvoke.net/default.aspx/gdi32/GetDeviceCaps.html
        }
        [DllImport("gdi32.dll")]
        private static extern int GetDeviceCaps(IntPtr hdc, int nIndex);

        [System.Runtime.InteropServices.DllImport("user32.dll")]
        internal static extern bool SetProcessDPIAware();

        [System.Runtime.InteropServices.DllImport("user32.dll")]
        internal static extern bool GetPhysicalCursorPos(ref CursorPoint lpPoint);

        public struct CursorPoint
        {
            public int X;
            public int Y;
        }

        public frmAttention()
        {
            InitializeComponent();
            pen_detected = false;
            this.ControlBox = false;
        }

        private void Subscribe(IKeyboardMouseEvents events)
        {
            m_Events = events;

            m_Events.MouseClick += OnMouseClick;
            m_Events.MouseDown += OnMouseDown;
            m_Events.MouseUp += OnMouseUp;
            /*
            m_Events.MouseDoubleClick += OnMouseDoubleClick;
            */

            m_Events.MouseMove += HookManager_MouseMove;
        }

        private void Unsubscribe()
        {
            if (m_Events == null) return;

            m_Events.MouseClick -= OnMouseClick;
            m_Events.MouseDown -= OnMouseDown;
            m_Events.MouseUp -= OnMouseUp;
            /*
            m_Events.MouseDoubleClick -= OnMouseDoubleClick;
            */

            m_Events.MouseMove -= HookManager_MouseMove;

            m_Events.Dispose();
            m_Events = null;
        }

        private void HookManager_MouseMove(object sender, MouseEventArgs e)
        {
            bool needToUpdateForm = false;
            int x = e.X;
            int y = e.Y;

            MouseEventExtArgs mxa = (MouseEventExtArgs)e;
            if (mxa.Timestamp == 0)
            {
                if (pen_detected == false)
                {
                    pen_detected = true;
                    needToUpdateForm = true;
                }
                // Start timer to undetect the pen
                timer1.Enabled = true;
                timer1.Interval = 1000;
            }
            else
                pen_detected = false;

            //Console.WriteLine(x + " " + y + " " + pen_detected);

            CursorPoint cursorPos = new CursorPoint();
            cursorPos.X = -1;
            cursorPos.Y = -1;
            try
            {
                GetPhysicalCursorPos(ref cursorPos);
                x = cursorPos.X;
                y = cursorPos.Y;
            }
            catch (EntryPointNotFoundException) // Not Windows Vista
            {
                //Console.WriteLine("Exception somewhere getting cursor pos");
                // Do nothing if error getting cursor position
                return;
            }
            //Console.WriteLine("GetCursorPos " + cursorPos.X + " " + cursorPos.Y);


            // Weird issue with doubled x/y coordinate after mousedown?  Try to ignore it
            if (ignoreClick == false)
            {
                ignoreClick = true;
                if ((x > lastX*1.5) && (y > lastY*1.5))
                {
                    // Do nothing
                    return;
                }
            }
            MouseMoved(x, y);

            // Show the form if it wasn't visible before
            if (needToUpdateForm)
                updateHighlightForm();

            /*          
            Log(string.Format("MouseEventSource {0}\n", mxa.Timestamp));    // 0 = pen or touch, non-zero = mouse
            labelMousePosition.Text = string.Format("x={0:0000}; y={1:0000}", x, y);
            if (highlightForm != null && highlightForm.Visible)
            {
                highlightForm.Left = x;
                highlightForm.Top = y;
            }
            */
        }

        private void OnMouseDown(object sender, MouseEventArgs e)
        {
            //Log(string.Format("MouseDown \t\t {0}\n", e.Button));
            //Console.WriteLine("Down detected");
            ignoreClick = false;  
            if (e.Button.ToString() == "Right")
            {
                toggleHighlightColors();
            }
        }

        private void OnMouseUp(object sender, MouseEventArgs e)
        {
            //Log(string.Format("MouseDown \t\t {0}\n", e.Button));
            //Console.WriteLine("Up detected");
            ignoreClick = false;
        }

        private void OnMouseClick(object sender, MouseEventArgs e)
        {
            //Log(string.Format("MouseClick \t\t {0}\n", e.Button));
        }

        private void frmSettings_Load(object sender, EventArgs e)
        {
            //SetProcessDPIAware();  // Allow window to scale

            //SetWindowLong(this.Handle, -20, (int)GetWindowLong(this.Handle, -20) | 0x00000020);

            //this.Left = System.Windows.Forms.Control.MousePosition.X -15;
            //this.Top = System.Windows.Forms.Control.MousePosition.Y - 15;

            //myHwnd = FindWindowByCaption(0, this.Text);
            //setMyHook(myHwnd);
            //Register the message
            lMsg = RegisterWindowMessage("UWM_MOUSEMOVE_MSG-PENATTEN-44E531B1_14D3_11d5_A025_006067718D04");
            Unsubscribe();
            Subscribe(Hook.GlobalEvents());

            //int osversion = System.Environment.OSVersion.Version.Major;
            //if (osversion >= 6)
            //{
                // Vista, add message to filter list so we can get it from IE
            //    ChangeWindowMessageFilter(Convert.ToUInt32(lMsg), 1);
            //}

            // Defaults for highlighting
            btnColor.BackColor = PenAttention.Properties.Settings.Default.highlightColor;
            cbHighlightMouse.Checked = PenAttention.Properties.Settings.Default.highlightMouse;
            tbRadius.Value = PenAttention.Properties.Settings.Default.highlightRadius;
            tbOpacity.Value = PenAttention.Properties.Settings.Default.highlightOpacity;
            circleHighlight = PenAttention.Properties.Settings.Default.circleHighlight;
            tbHeight.Value = PenAttention.Properties.Settings.Default.highlightHeight;
            tbWidth.Value = PenAttention.Properties.Settings.Default.highlightWidth;
            cbToggleColor.Checked = PenAttention.Properties.Settings.Default.toggleColor;
            highlightChecked = cbHighlightMouse.Checked; 

            if (circleHighlight)
            {
                rbCircle.Checked = true;
            }
            else
            {
                rbRectangle.Checked = true;
            }

            // if file exists, load the image in place of the pen icon
            if (File.Exists("CustomPen.png"))
            {
                pboxPencil.Image = new Bitmap("CustomPen.png");
            }

            // Hotkey
            hotkey = (byte)PenAttention.Properties.Settings.Default.hotKey;
            if (hotkey == HOTKEY_OFF)
                rbHotkeyOff.Checked = true;
            else if (hotkey == CTRL_ALT_F9)
            {
                rbCtrlAltF9.Checked = true;
                RegisterHotKey(this.Handle, this.GetType().GetHashCode(), 3, (int)Keys.F9);
            }
            else if (hotkey == CTRL_F9)
            {
                rbCtrlF9.Checked = true;
                RegisterHotKey(this.Handle, this.GetType().GetHashCode(), 2, (int)Keys.F9);
            }

            // Icon or highlight
            int attentionMode = PenAttention.Properties.Settings.Default.selectedIcon;
            if (attentionMode == 0)
            {
                attention_type = Attention_Mode.Highlight;
                this.showHighlightToolStripMenuItem.Checked = true;
                highlightChecked = true;
                setHighlightForm();
                updateHighlightForm();
            }
            else if (attentionMode == 1)
            {
                attention_type = Attention_Mode.Pencil;
                this.showPencilToolStripMenuItem.Checked = true;
                highlightChecked = false; pencilChecked = true;
                setIconForm(pboxPencil);
                updateHighlightForm();
            }
            else if (attentionMode == 2)
            {
                attention_type = Attention_Mode.Pointer;
                this.showPointerToolStripMenuItem.Checked = true;
                highlightChecked = false; pointerChecked = true;
                setIconForm(pboxPointer);
                updateHighlightForm();
                //highlightForm.Hide();
            }
            else if (attentionMode == -1)
            {
                attention_type = Attention_Mode.None;
                highlightForm.Hide();
            }
            //Hide();
        }

        private void setHighlightForm()
        {
            if (circleHighlight)
            {
                curBmp = new Bitmap(tbRadius.Value * 2, tbRadius.Value * 2);
                Graphics g = Graphics.FromImage(curBmp);
                SolidBrush b = new SolidBrush(btnColor.BackColor);
                g.FillEllipse(b, 0, 0, tbRadius.Value * 2, tbRadius.Value * 2);

                highlightForm.SetBitmap(curBmp, (byte)(tbOpacity.Value * 255 / 100));
                highlightForm.Show();
                DrawPrimaryForm(System.Windows.Forms.Control.MousePosition.X - tbRadius.Value,
                                System.Windows.Forms.Control.MousePosition.Y - tbRadius.Value);
            }
            else
            {
                curBmp = new Bitmap(tbWidth.Value, tbHeight.Value);
                Graphics g = Graphics.FromImage(curBmp);
                SolidBrush b = new SolidBrush(btnColor.BackColor);
                g.FillRectangle(b, 0, 0, tbWidth.Value, tbHeight.Value);

                highlightForm.SetBitmap(curBmp, (byte)(tbOpacity.Value * 255 / 100));
                highlightForm.Show();
                DrawPrimaryForm(System.Windows.Forms.Control.MousePosition.X - (tbWidth.Value / 2),
                                System.Windows.Forms.Control.MousePosition.Y - (tbHeight.Value / 2));
            }
        }

        private void setIconForm(PictureBox p)
        {
            curBmp = new Bitmap(p.Image);

            highlightForm.SetBitmap(curBmp, (byte)254);
            highlightForm.Show();
            DrawPrimaryForm(System.Windows.Forms.Control.MousePosition.X - tbRadius.Value,
                            System.Windows.Forms.Control.MousePosition.Y - tbRadius.Value);
        }

        public void MouseMoved(int X, int Y)
        {            
            // If no pen, do normal mouse thing
            if (!pen_detected  && !cbHighlightMouse.Checked)
            {
                return;
            }

            // If no attention mode, do nothing
            if (attention_type == Attention_Mode.None)
            {
                return;
            }

            // Remember last coordinates
            lastX = X;
            lastY = Y;
            ignoreClick = true;

            // Pen detected so draw attention to the mouse location appropriately
            if (attention_type == Attention_Mode.Highlight)
            {
                DrawPrimaryForm(X - curBmp.Width / 2, Y - curBmp.Height / 2);
            }
            else
            {
                if (cbHighlightMouse.Checked || pen_detected)
                {
                    //Console.WriteLine("Should draw pencil? " + pen_detected);
                    DrawPrimaryForm(X, Y);
                }
            }
        }

        // Draw the cursor or highlight on the primary screen
        private void DrawPrimaryForm(int X, int Y)
        {
            highlightForm.Left = X;
            highlightForm.Top = Y;
        }

        protected override void WndProc(ref Message m)
        {
            //calling the base first is important, otherwise the values you set later will be lost
            base.WndProc(ref m);

            /*
            if (m.Msg == lMsg)
            {
                if (m.WParam.ToInt32() == RCLICKCODE)       // Right click
                {
                    toggleHighlightColors();
                }                
                else if (m.WParam.ToInt32() == KEYTOGGLE)   // On/Off keyboard toggle
                {

                }
                else                                        // Mouse Move
                {
                    MouseMoved(m.WParam.ToInt32(), m.LParam.ToInt32());
                }
                //Console.WriteLine(m.WParam + " " + m.LParam + ": " + System.Windows.Forms.Cursor.Position.X + "," + System.Windows.Forms.Cursor.Position.Y);
            }
        */
            if (m.Msg == KEYTOGGLE)
            {
                // If nothing is checked now, then turn on whatever used to be selected
                if (nothingChecked())
                {
                    if (highlightChecked)
                    {
                        showHighlightToolStripMenuItem.Checked = true;
                        showSettingsToolStripMenuItem_Click(this, EventArgs.Empty);
                    }
                    else if (pencilChecked)
                    {
                        showPencilToolStripMenuItem.Checked = true;
                        showPencilToolStripMenuItem_Click(this, EventArgs.Empty);
                    }
                    else if (pointerChecked)
                    {
                        showPointerToolStripMenuItem.Checked = true;
                        showPointerToolStripMenuItem_Click(this, EventArgs.Empty);
                    }
                }
                // If something is checked, remember it and turn the check off
                else
                {
                    highlightChecked = false;
                    pencilChecked = false;
                    pointerChecked = false;
                    if (showHighlightToolStripMenuItem.Checked)
                    {
                        highlightChecked = true;
                        showHighlightToolStripMenuItem.Checked = false;
                        showSettingsToolStripMenuItem_Click(this, EventArgs.Empty);
                    }
                    if (showPencilToolStripMenuItem.Checked)
                    {
                        pencilChecked = true;
                        showPencilToolStripMenuItem.Checked = false;
                        showPencilToolStripMenuItem_Click(this, EventArgs.Empty);
                    }
                    if (showPointerToolStripMenuItem.Checked)
                    {
                        pointerChecked = true;
                        showPointerToolStripMenuItem.Checked = false;
                        showPointerToolStripMenuItem_Click(this, EventArgs.Empty);
                    }
                }
            }
        }

        // Toggle highlight colors if in highlight mode and option checked
        private void toggleHighlightColors()
        {
            if ((attention_type == Attention_Mode.Highlight) && cbToggleColor.Checked)
            {
                Color c = btnColor.BackColor;
                byte r = c.R;
                byte g = c.G;
                byte b = c.B;
                btnColor.BackColor = Color.FromArgb(b, r, g);
                setHighlightForm();
            }
        }

        private void frmSettings_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Uninitialize hooks
            //actHook.Stop();
            //clearMyHook(myHwnd);
            Unsubscribe();
        }

    private void frmSettings_Resize(object sender, EventArgs e)
        {
            if (FormWindowState.Minimized == WindowState)
                Hide();
        }

        private void notifyIcon1_DoubleClick(object sender, EventArgs e)
        {
            contextMenuStrip1.Show();
            //Show();
            //WindowState = FormWindowState.Normal;
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private bool nothingChecked()
        {
            if (!showHighlightToolStripMenuItem.Checked && !showPencilToolStripMenuItem.Checked && !showPointerToolStripMenuItem.Checked)
                return true;
            return false;
        }

        private void showSettingsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (showHighlightToolStripMenuItem.Checked)
            {
                // Undo other checkmarks
                showPencilToolStripMenuItem.Checked = false;
                showPointerToolStripMenuItem.Checked = false;

                attention_type = Attention_Mode.Highlight;
                PenAttention.Properties.Settings.Default.selectedIcon = 0;
                PenAttention.Properties.Settings.Default.Save();

                // Highlight the pen option
                setHighlightForm();
                updateHighlightForm();
            }
            else
            {
                if (nothingChecked())
                {
                    attention_type = Attention_Mode.None;
                    PenAttention.Properties.Settings.Default.selectedIcon = -1;
                    PenAttention.Properties.Settings.Default.Save();
                    highlightForm.Hide();
                }
            }
        }

        private void showPencilToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (showPencilToolStripMenuItem.Checked)
            {
                // Undo other checkmarks
                showHighlightToolStripMenuItem.Checked = false;
                showPointerToolStripMenuItem.Checked = false;

                attention_type = Attention_Mode.Pencil;
                PenAttention.Properties.Settings.Default.selectedIcon = 1;
                PenAttention.Properties.Settings.Default.Save();

                // Show Pencil option
                setIconForm(pboxPencil);
                updateHighlightForm();
                MouseMoved(System.Windows.Forms.Control.MousePosition.X, System.Windows.Forms.Control.MousePosition.Y);
            }
            else
            {
                if (nothingChecked())
                {
                    attention_type = Attention_Mode.None;
                    PenAttention.Properties.Settings.Default.selectedIcon = -1;
                    PenAttention.Properties.Settings.Default.Save();
                    highlightForm.Hide();
                }
            }
        }

        private void showPointerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (showPointerToolStripMenuItem.Checked)
            {
                // Undo other checkmarks
                showHighlightToolStripMenuItem.Checked = false;
                showPencilToolStripMenuItem.Checked = false;

                attention_type = Attention_Mode.Pointer;
                PenAttention.Properties.Settings.Default.selectedIcon = 2;
                PenAttention.Properties.Settings.Default.Save();
                
                // Show pointer option
                setIconForm(pboxPointer);
                updateHighlightForm();
                MouseMoved(System.Windows.Forms.Control.MousePosition.X, System.Windows.Forms.Control.MousePosition.Y);
            }
            else
            {
                if (nothingChecked())
                {
                    attention_type = Attention_Mode.None;
                    PenAttention.Properties.Settings.Default.selectedIcon = -1;
                    PenAttention.Properties.Settings.Default.Save();
                    highlightForm.Hide();
                }
            }
        }

        private void updateHighlightForm()
        {
            if (attention_type == Attention_Mode.None)
            {
                highlightForm.Hide();
                return;
            }

            // If not tracking the mouse, then if no attention mode, or no pen, do nothing
            if (!cbHighlightMouse.Checked)
            {
                if (attention_type == Attention_Mode.None || pen_detected == false)
                {
                    if (highlightForm.Visible)
                        highlightForm.Hide();
                    return;
                }
            }
            // If the attention mode is not highlight (it's pencil or pointer)
            // then hide the form if no pen is detected
            if ((attention_type == Attention_Mode.Pencil) || (attention_type == Attention_Mode.Pointer))
            {
                if ((!pen_detected) && (!cbHighlightMouse.Checked))
                {
                    highlightForm.Hide();
                    return;
                }
            }

            if (!highlightForm.Visible)
            {
                highlightForm.Show();
            }
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            origColor = btnColor.BackColor;
            origHighlightMouse = cbHighlightMouse.Checked;
            origOpacity = tbOpacity.Value;
            origRadius = tbRadius.Value;
            origCircleHighlight = rbCircle.Checked;
            origHeight = tbHeight.Value;
            origWidth = tbWidth.Value;
            origToggleColor = cbToggleColor.Checked;
            origHotkey = hotkey;

            made_changes = false;
            Show();
            WindowState = FormWindowState.Normal;
        }

        private void notifyIcon1_Click(object sender, EventArgs e)
        {
            int x  = System.Windows.Forms.Cursor.Position.X;
            int y = System.Windows.Forms.Cursor.Position.Y - 40;
            contextMenuStrip1.Show( x, y-100);
        }

        private void aboutToolStripMenuItem_Click_1(object sender, EventArgs e)
        {
            frmAbout about = new frmAbout();
            about.ShowDialog();
        }

        private void tbOpacity_Scroll(object sender, EventArgs e)
        {
            lblOpacity.Text = tbOpacity.Value.ToString();
            pboxSample.Invalidate();
            made_changes = true;
        }

        private void btnColor_Click(object sender, EventArgs e)
        {
            ColorDialog cd = new ColorDialog();
            cd.ShowDialog();

            btnColor.BackColor = cd.Color;
            pboxSample.Invalidate();
            made_changes = true;
        }

        private void pboxSample_Paint(object sender, PaintEventArgs e)
        {
            Graphics gr = e.Graphics;
            int r, g, b;

            r = btnColor.BackColor.R;
            g = btnColor.BackColor.G;
            b = btnColor.BackColor.B;
            Brush br = new SolidBrush(Color.FromArgb(tbOpacity.Value*255/100, r,g,b));
            if (circleHighlight)
            {
                gr.FillEllipse(br, 80 - tbRadius.Value, 80 - tbRadius.Value, tbRadius.Value * 2, tbRadius.Value * 2);
            }
            else
            {
                int clipWidth = tbWidth.Value;
                int clipHeight = tbHeight.Value;
                gr.FillRectangle(br, 80 - (clipWidth / 2), 80 - (clipHeight / 2), clipWidth, clipHeight);         
            }
        }

        private void btnApply_Click(object sender, EventArgs e)
        {
            PenAttention.Properties.Settings.Default.highlightColor = btnColor.BackColor;
            PenAttention.Properties.Settings.Default.highlightMouse = cbHighlightMouse.Checked;
            PenAttention.Properties.Settings.Default.highlightOpacity = tbOpacity.Value;
            PenAttention.Properties.Settings.Default.highlightRadius = tbRadius.Value;
            PenAttention.Properties.Settings.Default.circleHighlight = rbCircle.Checked;
            PenAttention.Properties.Settings.Default.highlightWidth = tbWidth.Value;
            PenAttention.Properties.Settings.Default.highlightHeight = tbHeight.Value;
            PenAttention.Properties.Settings.Default.Save();

            // Force selection to Highlight if we click apply
            showHighlightToolStripMenuItem.Checked = true;
            showSettingsToolStripMenuItem_Click(sender, e);

            setHighlightForm();
            updateHighlightForm();
            pboxSample.Invalidate();
            made_changes = false;
        }

        private void btnRevert_Click(object sender, EventArgs e)
        {
            tbOpacity.Value = origOpacity;
            tbRadius.Value = origRadius;
            btnColor.BackColor = origColor;
            cbHighlightMouse.Checked = origHighlightMouse;
            circleHighlight = origCircleHighlight;
            if (circleHighlight)
                rbCircle.Checked = true;
            else
                rbRectangle.Checked = true;
            tbWidth.Value = origWidth;
            tbHeight.Value = origHeight;
            cbToggleColor.Checked = origToggleColor;
            hotkey = origHotkey;
            if (hotkey == HOTKEY_OFF)
            {
                rbHotkeyOff.Checked = true;
                UnregisterHotKey(this.Handle, this.GetType().GetHashCode());
            }
            else if (hotkey == CTRL_ALT_F9)
            {
                rbCtrlAltF9.Checked = true;
                RegisterHotKey(this.Handle, this.GetType().GetHashCode(), 3, (int)Keys.F9);
            }
            else if (hotkey == CTRL_F9)
            {
                rbCtrlF9.Checked = true;
                RegisterHotKey(this.Handle, this.GetType().GetHashCode(), 2, (int)Keys.F9);
            }

            setHighlightForm();
            updateHighlightForm();
            pboxSample.Invalidate();
            made_changes = false;
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            if (made_changes)
            {
                DialogResult dr = MessageBox.Show("Your settings were modified. Save changes?", "Save Changes", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (dr == DialogResult.Yes)
                {
                    btnApply_Click(sender, e);
                }
                else
                {
                    btnRevert_Click(sender, e);
                }
            }
            Hide();
        }

        private void cbHighlightMouse_CheckedChanged(object sender, EventArgs e)
        {
            made_changes = true;
            setHighlightForm();
            updateHighlightForm();
        }

        private void tbRadius_Scroll_1(object sender, EventArgs e)
        {
            lblRadius.Text = tbRadius.Value.ToString();
            pboxSample.Invalidate();
            made_changes = true;
        }

        private void rbRectangle_CheckedChanged(object sender, EventArgs e)
        {
            circleHighlight = false;
            pboxSample.Invalidate();
            made_changes = true;
        }

        private void rbCircle_CheckedChanged(object sender, EventArgs e)
        {
            circleHighlight = true;
            pboxSample.Invalidate();
            made_changes = true;
        }

        private void tbWidth_Scroll(object sender, EventArgs e)
        {
            lblWidth.Text = tbWidth.Value.ToString();
            pboxSample.Invalidate();
            made_changes = true;
        }

        private void tbHeight_Scroll(object sender, EventArgs e)
        {
            lblHeight.Text = tbHeight.Value.ToString();
            pboxSample.Invalidate();
            made_changes = true;
        }

       private void cbToggleColor_CheckedChanged(object sender, EventArgs e)
        {
            PenAttention.Properties.Settings.Default.toggleColor = cbToggleColor.Checked;
            PenAttention.Properties.Settings.Default.Save();
        }

        private void rbHotkeyOff_CheckedChanged(object sender, EventArgs e)
        {
            if (rbHotkeyOff.Checked)
            {
                hotkey = HOTKEY_OFF;
                UnregisterHotKey(this.Handle, this.GetType().GetHashCode());
                PenAttention.Properties.Settings.Default.hotKey = hotkey;
                PenAttention.Properties.Settings.Default.Save();
            }
        }

        private void rbCtrlF9_CheckedChanged(object sender, EventArgs e)
        {
            if (rbCtrlF9.Checked)
            {
                hotkey = CTRL_F9;
                RegisterHotKey(this.Handle, this.GetType().GetHashCode(), 2, (int)Keys.F9);
                PenAttention.Properties.Settings.Default.hotKey = hotkey;
                PenAttention.Properties.Settings.Default.Save();
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            // Timer went off from detecting the pen
            timer1.Enabled = false;
            // Assume pen is no longer near the surface
            pen_detected = false;
            updateHighlightForm();
        }

        private void rbCtrlAltF9_CheckedChanged(object sender, EventArgs e)
        {
            if (rbCtrlAltF9.Checked)
            {
                hotkey = CTRL_ALT_F9;
                RegisterHotKey(this.Handle, this.GetType().GetHashCode(), 3, (int)Keys.F9);
                PenAttention.Properties.Settings.Default.hotKey = hotkey;
                PenAttention.Properties.Settings.Default.Save();
            }
        }

        private void frmAttention_FormClosing(object sender, FormClosingEventArgs e)
        {
            UnregisterHotKey(this.Handle, this.GetType().GetHashCode());
        }

        /*
        // Scaling factor of primary screen
        private float getScalingFactor()
        {
            Graphics g = Graphics.FromHwnd(IntPtr.Zero);
            IntPtr desktop = g.GetHdc();
            int LogicalScreenHeight = GetDeviceCaps(desktop, (int)DeviceCap.VERTRES);
            int PhysicalScreenHeight = GetDeviceCaps(desktop, (int)DeviceCap.DESKTOPVERTRES);

            float ScreenScalingFactor = (float)PhysicalScreenHeight / (float)LogicalScreenHeight;

            return ScreenScalingFactor; // 1.25 = 125%
        }
        */

    }
}